% FullBNT musi byt na Matlab ceste

% ocislovani uzlu (NB: uzly musi byt ocislovane v topologickem usporadani)
RM = 1; PN = 2; SS = 3; PV = 4; PS = 5;

% nazvy uzlu
names = {'rm','pn','ss','pv','ps'};

% priprava matice primych podminenych zavislosti
dag = zeros(5,5);

% zadani primych podminenych zavislosti tj. hran site
dag(RM,SS)=1;
dag([RM PN],PV)=1;
dag(PV,PS) = 1;

% zobrazeni grafu site 
draw_graph(dag, names);

% zadani velikosti jednotlivych uzlu tj. kolik ruznych hodnot muze dany
% uzel mit (v nasem prikladu jsou vsechny uzly binarni)
node_sizes = 2*ones(1,5);

% vytvoreni kostry site
bnet = mk_bnet(dag, node_sizes, 'names', names);

% k uzlum pak muzeme pristupovat pomoci jejich nazvu napr.:
RM = bnet.names('rm');

% zadani parametru site prostrednictvim tabulek podminenych pravdepodobnosti
% prvni hodnotu povazuji za T (true~1), druhou za F (false~2) -- nutno konzistentne dodrzet v cele uloze
% (v BNT nelze zavest obvyklou notaci T~1, F~0, pouziva indexy 1 a 2)
bnet.CPD{RM} = tabular_CPD(bnet, RM, 'CPT', [0.15 0.85]');
bnet.CPD{PN} = tabular_CPD(bnet, PN, 'CPT', [0.01 0.99]');

% RM SS prob
% ----------
% T  T   0.6   P(SS=T|RM=T) = 0.6
% F  T   0.05  P(SS=T|RM=F) = 0.05
% T  F   0.4   P(SS=F|RM=T) = 0.4
% F  F   0.05  P(SS=F|RM=F) = 0.95
bnet.CPD{SS} = tabular_CPD(bnet, SS, 'CPT', [0.6 0.05 0.4 0.95]');

% RM PN PV prob
% --------------
% T  T  T  0.99   P(PV=T|RM=T,PN=T) = 0.99
% F  T  T  0.97   P(PV=T|RM=F,PN=T) = 0.97
% T  F  T  0.9    P(PV=T|RM=T,PN=F) = 0.9
% F  F  T  0.3    P(PV=T|RM=F,PN=F) = 0.3
% T  T  F  0.01   P(PV=F|RM=T,PN=T) = 0.01
% F  T  F  0.03   P(PV=F|RM=F,PN=T) = 0.03
% T  F  F  0.1    P(PV=F|RM=T,PN=F) = 0.1
% F  F  F  0.7    P(PV=F|RM=F,PN=F) = 0.7
bnet.CPD{PV} = tabular_CPD(bnet, PV, 'CPT', [0.99 0.97 0.9 0.3 0.01 0.03 0.1 0.7]');
bnet.CPD{PS} = tabular_CPD(bnet, PS, 'CPT', [0.7 0.01 0.3 0.99]');

% prirazeni inferencniho stroje
% pouziva variable elimination
engine = jtree_inf_engine(bnet); 
% jsou i jine moznosti, napr. pearl ... pouziva belief propagation a uklada informace o iteracich do bel.txt 
% engine = pearl_inf_engine(bnet,'filename','bel.txt'); 

% pocitani marginalnich pravdepodobnosti na zaklade pozorovani

% zadani pozorovani - hodnoty uzlu reprezentovany cisly tj. T ~ 1, F ~ 2
% (viz definice CPTs vyse)
ev = cell(1,5);
ev{PS} = 2; % pes nesteka tj. PS = F
ev{SS} = 1; % svetlo sviti tj. SS = T
engine = enter_evidence(engine, ev);

% pocitani podminene pravdepodobnosti
m = marginal_nodes(engine, RM);
fprintf('Pr(RM=T|PS=F,SS=T) = %5.4f\n', m.T(1))
fprintf('Pr(RM=F|PS=F,SS=T) = %5.4f\n', m.T(2))

% pocitani sdruzene pravdepodobnosti -- uplna tabulka
% nelze kazdym inf. strojem, 
% LW vzorkuje -- priblizny odhad, lze zpresnit navysenim parametru nsamples
engine = likelihood_weighting_inf_engine(bnet,'nsamples',1000);
ev = cell(1,5);
engine = enter_evidence(engine, ev);
m = marginal_nodes(engine, [RM PN SS PV PS]);
% Pr(RM=T,PN=F,SS=F,PV=T,PS=T) = 0.15*0.99*0.4*0.9*0.7=0.0374
fprintf('Pr(RM=T,PN=F,SS=F,PV=T,PS=T) = %5.4f\n', m.T(1,2,2,1,1))
% Pr(RM=F,PN=F,SS=F,PV=T,PS=T) = 0.85*0.99*0.95*0.3*0.7=0.1679
fprintf('Pr(RM=F,PN=F,SS=F,PV=T,PS=T) = %5.4f\n', m.T(2,2,2,1,1))

